/*
 * file: pv_list.h
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#ifndef PV_LIST_H
#define PV_LIST_H

struct pv_list {
	struct pv *head;
	struct pv *tail;
	struct pv *cur;
};

struct jv_varlist {
	struct jvisu_var *head;
	struct jvisu_var *tail;
	struct jvisu_var *cur;
};

#define goto_head(l)			\
	((*(l))->cur = (*(l))->head)

#define goto_tail(l)			\
	(*(l))->cur = (*(l))->tail

#define goto_next(l)			\
	(*(l))->cur = (*(l))->cur->next

#define list_add_tail(l, d, size)	\
do {					\
	typeof((l)->cur) tmp;		\
	tmp = malloc(size);		\
	memcpy(tmp, d, size);		\
	if ((l)->tail)			\
		(l)->tail->next = tmp;	\
	else				\
		(l)->head = tmp;	\
	tmp->next = NULL;		\
	(l)->tail = tmp;		\
} while (0);

#define list_init(l)			\
do {					\
	(l) = calloc(1, sizeof(*(l)));	\
	(l)->head = NULL;		\
	(l)->tail = NULL;		\
	(l)->cur = NULL;		\
} while (0);

#define list_del_element(l)		\
do {					\
	typeof((l)->cur) e1, e2;	\
	e1 = NULL;			\
	e2 = (l)->head;			\
	while ((e2 != (l)->cur) && e2)	\
	{				\
		e1 = e2;		\
		e2 = e2->next;		\
	}				\
	if (!e2) break;			\
	if (e1)				\
		e1->next = e2->next;	\
	else				\
		(l)->head = e2->next;	\
	(l)->cur = e2->next ? e2->next:e1;\
	free(e2);			\
} while (0);

#define list_del(l)			\
do {					\
while ((l)->head) {			\
	(l)->cur = (l)->head;		\
	(l)->head = (l)->head->next;	\
	free((l)->cur);			\
}					\
free(l);				\
} while (0);

#define is_last(l)			\
	(!(l)->cur || (l)->cur->next == 0)

#define is_empty(l)			\
	((!l) || ((l)->head == 0))


#define list_get_element_data(l)	\
	((l)->cur)

#endif	/* !PV_LIST_H */
