/*
 * file: pv_register.h
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#ifndef PV_REGISTER_H
#define PV_REGISTER_H

#include <pv_types.h>

/*
 * calculate an absolute pointer to a process variable
 * (given by offset) into the current shared memory
 */
static inline union pv_data
*pv_to_abs_addr(const struct pv_engine	*pv_engine,
		const struct pv		*pv)
{
	return (union pv_data *)(pv_engine->store.map + pv->offset);
}

/*
 * calculate an offset into the current shared memory from
 * an absolute pointer to the same process variable
 * FIXME: What if the given pointer is not a process variable pointer?
 * Or not one of this engine?
 */
static inline off_t
abs_addr_to_offset(const struct pv_engine *pv_engine,
		const union pv_data *pv)
{
	return (off_t)((void*)pv - (void*)pv_engine->store.map);
}

struct pv
*pv_register_lookup(struct pv_engine	*pv_engine,
		    const char		*id);

#endif	/* !PV_REGISTER_H */
